;=============================================================================
; File name:  STM8_TSL_RC_routines.asm
;
; This file contains some assembly routines for the touch-sensing ST library.
;-----------------------------------------------------------------------------
; Notes:
; - The assembly routines available below must be WORD-aligned for speed.
; - If your project uses the large model using the RCSTM8 directive STM8(LARGE),
;   functions will be called using CALLF, and will return using RETF. For the
;   following code to work properly, you must set the LARGE macro to 1 so that
;   the proper CALL/RET scheme will be used.
; - Each routine has its own segment, so that it is easier to align them on
;   even addresses.
;-----------------------------------------------------------------------------
; (c) Copyright Raisonance S.A.S. 2009-2010, All rights reserved.
;=============================================================================

$MODESTM8
;$SET(LARGE = 1)          ; Uncomment if you are in LARGE mode

$IF(LARGE==1)
 BIBRET  MACRO
    RETF
 ENDM
$ELSE
 BIBRET  MACRO
    RET
 ENDM
$ENDIF


;-----------------------------------------------------------------------------
; Public symbols. These are the assembly routines exported to C.
;-----------------------------------------------------------------------------

        PUBLIC  ?TSL_IO_SW_Burst_TestSyncShift
        PUBLIC  ?TSL_IO_SW_Burst_Wait_Vil
        PUBLIC  ?TSL_IO_SW_Burst_Wait_Vih
        PUBLIC  ?TSL_IO_SW_Burst_Stop_Timer


;-----------------------------------------------------------------------------
; Externs. These are the variables defined in C modules accessed in assembly.
;-----------------------------------------------------------------------------

        EXTRN   DATA0(SamplingShifter)
        EXTRN   DATA0(AcquisitionBitMask)
        EXTRN   DATA0(sTouchIO)
        EXTRN   DATA0(CounterStop)

        EXTRN   DATA(TIMACQ_CNTR)


        STM8_TSL_TST        SEGMENT CODE INSECTION WORD
        RSEG                STM8_TSL_TST

;=============================================================================
; Execute a loop for calibrated number of cycles to allow over sampling
; Warning: This function must be aligned in memory (start at an even address).
;=============================================================================

?TSL_IO_SW_Burst_TestSyncShift:

        LD      A, SamplingShifter

        ; 3 cycles loop if jump executed; 2 cycles loop if not.
_ShiftLoopVih:
        DEC     A                       ; 1 cycle
        JRNE    _ShiftLoopVih

        BIBRET



        STM8_TSL_VIL        SEGMENT CODE INSECTION WORD
        RSEG                STM8_TSL_VIL

;=============================================================================
; Local sub function to wait for vil on RC structure.
; Warning: This function must be aligned in memory (start at an even address).
;=============================================================================

?TSL_IO_SW_Burst_Wait_Vil:

        NOP
        LD      A, AcquisitionBitMask
        LDW     X, sTouchIO             ; Input data register ...
        INCW    X

        ; Loop = 1 + 1 + 2 + 2 + 2 cycles = 8 cycles
_WaitForVil:
        BCP     A, (X)                  ; 1 cycles
        JREQ    _EndWaitForVil
        LDW     Y, TIMACQ_CNTR          ; 2 cycles; hw counter also used for timeout ...
        CPW     Y, #00E00h              ; 2 cycles; Timeout compare
        JRULT   _WaitForVil
_EndWaitForVil:

        BIBRET


        STM8_TSL_VIH        SEGMENT CODE INSECTION WORD
        RSEG                STM8_TSL_VIH

;=============================================================================
; Local sub function to wait for vih on RC structure.
; Warning: This function must be aligned in memory (start at an even address).
;=============================================================================

?TSL_IO_SW_Burst_Wait_Vih:

        NOP
        LD      A, AcquisitionBitMask
        LDW     X, sTouchIO             ; Input data register ...
        INCW    X

        ; Loop = 1 + 1 + 2 + 2 + 2 cycles = 8 cycles
_WaitForVih:
        BCP     A, (X)                  ; 1 cycles
        JRNE    _EndWaitForVih
        LDW     Y, TIMACQ_CNTR          ; 2 cycles; hw counter also used for timeout ...
        CPW     Y, #00E00h              ; 2 cycles; Timeout compare
        JRULT   _WaitForVih
_EndWaitForVih:

        BIBRET


        STM8_TSL_STP        SEGMENT CODE INSECTION WORD
        RSEG                STM8_TSL_STP

;=============================================================================
; Local sub function to get timer count for acquisition.
; Warning: This function must be aligned in memory (start at an even address).
;=============================================================================

?TSL_IO_SW_Burst_Stop_Timer:

        LD      A, TIMACQ_CNTR
        LD      CounterStop, A
        LD      A, TIMACQ_CNTR + 1
        LD      CounterStop + 1, A

        BIBRET


END
